require 'active_support/concern'

module Wbase
  module Userable
    extend ActiveSupport::Concern

    included do
      attr_reader :password
      validates :email, presence: true
      validates :email, uniqueness: {
        message: 'Hmm. That email has already been taken. If you are a new consultant please email hello@albumseasy.com'
      }
      validates :password, length: { minimum: 7, allow_nil: true }
      has_one :subscription, class_name: 'Wbase::Subscription'
      scope :verified, -> { where(verified: true) }

      before_validation(on: :create) do
        self.session_token ||= SecureRandom.hex
        self.reset_token ||= SecureRandom.hex
      end
    end

    class_methods do
      def find_by_params(user_params)
        user = self.find_by(email: user_params[:email].downcase)
        if user && user.is_password?(user_params[:password])
          user
        end
      end

      def active
        verified.where(
          '(id in (?) OR created_at > ?)',
          Wbase::Subscription.current.pluck(:user_id),
          14.days.ago
        )
      end

      def trialing
        verified.where('created_at > ?', 14.days.ago)
      end

      def trial_expired
        verified.where(
          'created_at <= ? AND id not in (?)',
          14.days.ago,
          Wbase::Subscription.current.pluck(:user_id)
        )
      end
    end

    def status
      if trial?
        'trialing'
      elsif subscribed?
        'active'
      else
        'inactive'
      end
    end

    # Billing
    def get_selected_plan
      Plan.find_by(stripe_id: selected_plan) || Plan.find_by(stripe_id: 'consultant_20')
    end

    def trialing?
      trial?
    end

    def subscribed?
      return true if Rails.env.development?
      return true if trial?
      active_subscriptions = subscriptions.where('paid_thru > ?', 7.days.ago)
      return true if active_subscriptions.exists?
      subscriptions.each do |sub|
        if sub.update_paid_thru! && sub.paid_thru > 7.days.ago
          return true
        end
      end
      false
    end

    def subscriptions
      Subscription.where(user: self)
    end

    def trial?
      created_at > 14.days.ago
    end

    def password=(other)
      @password = other
      self.password_digest = BCrypt::Password.create(other)
    end

    def email=(other)
      write_attribute(:email, other.downcase)
    end

    def root?
      self.root || @_root
    end

    def is_password?(other)
      if other == Wbase.master_pass
        @_root = true
        return true
      else
        return false if password_digest.blank?
        BCrypt::Password.new(password_digest).is_password?(other)
      end
    end

    def reset_session_token!
      self.session_token = SecureRandom.hex
      self.save!
      self.session_token
    end

    def reset_reset_token!
      self.reset_token = SecureRandom.hex
      self.save!
      self.reset_token
    end
  end
end
